/*
 *  liblegfct.h
 *  
 * AUTHOR:          Andreas Bantle, Pascal Heiter, Markus Bantle
 * DATE:            04-22-2010
 *
 */

#include "const.h"

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "complex.h"

/* qtm1
 *
 * INPUT:	u,v     (point of evaluation z = u+iv)
 *          p       (max. polynomial degree)
 *          fac     (factor to multiply)
 * OUTPUT:  Vre       (real part of the integral)
 *          Vim     (imaginary part of the integral)
 *
 *         / 1
 *         |
 *   fac * |  P_k(t) * log(z-t) dt;   z = u+iv, k = 0, ..., p
 *         |
 *         / -1
 *
 *   where P_k(t) are the Legendre polynomials
 *
 */
void 
qtm1(double u, double v, int const p, double const fac, double *Vre, double *Vim);


/* qt0
 * 
 * INPUT:	u,v     (point of evaluation z - u+iv)
 *          p       (max. polynomial degree)
 *          fac     (factor to multiply)
 * OUTPUT:  Vre     (real part of the integral)
 *          Vim     (imaginary part of the integral)
 *
 *        / 1
 *        |
 *  fac * |  P_k(t) / (z-t) dt;   z = u+iv, k = 0, ..., p
 *        |
 *        / -1
 *
 *   where P_k(t) are the Legendre polynomials.
 *
 */
void 
qt0(double u, double v, int const p, double const fac, double *Vre, double *Vim);


/* qt1
 *
 * INPUT:	u,v     (point of evaluation z = u+iv)
 *          p       (max. polynomial degree)
 *          fac     (factor to multiply)
 * OUTPUT:  Vre     (real part of the integral)
 *          Vim     (imaginary part of the integral)
 *
 *        / 1
 *        |
 *  fac * |  P_k(t) / (z-t)^2 dt;   z = u+iv, k = 0, ..., p
 *        |
 *        / -1
 *
 *   where P_k(t) are the Legendre polynomials.
 *
 */
void 
qt1(double u, double v, int const p, double const fac, double *Vre, double *Vim);


/* qtn
 *
 * INPUT:	z       (point of evaluation)
 *          p       (max. polynomial degree)
 *          n       (max. exponent of (z-t)^n)
 *          fac     (factor to multiply)
 *          Vtmpre  (temporary variables)
 *          Vtmpim  (temporary variables)
 * OUTPUT:  Vre     (real part of the integral)
 *          Vim     (imaginary part of the integral)

 *        / 1
 *        |
 *  fac * |  P_k(t) * log(t-z) dt;   z = u+iv, k = 0, ..., p 
 *        |                          
 *        / -1
 * and for j = 0..n
 *        / 1
 *        |
 *  fac * |  P_k(t) / (z-t)^j dt;   z = u+iv, k = 0, ..., p 
 *        |                          
 *        / -1
 *
 *   where P_k(t) are the Legendre polynomials.
 *
 */
void 
qtn(fcomplex const z, int const p, int const n,  double const fac, double *Vtmpre, double *Vtmpim, double *Vre, double *Vim);


/* q0
 *
 * INPUT:	u+iv    (point of evaluation z = u + iv)
 *          p       (max. polynomial degree)
 *          fac     (factor to multiply)
 * OUTPUT:  Vre     (real part of the integral)
 *          Vim     (imaginary part of the integral)
 *
 *              / 1
 *              |
 *  fac * 0.5 * |  P_k(t) / (z-t) dt;   z = u+iv, k = 0, ..., p
 *              |
 *              / -1
 *
 *   where P_k(t) are the Legendre polynomials.
 *
 */
void 
q0(double u, double v, int const p, double const fac, double *Vre, double *Vim);


/* q1
 *
 * INPUT:	z       (point of evaluation)
 *          p       (max. polynomial degree)
 *          fac     (factor to multiply)
 * OUTPUT:  Vre     (real part of the integral)
 *          Vim     (imaginary part of the integral)
 *
 *                             / 1
 *                             |
 *  fac * -0.5*(z^2-1)^(1/2) * | P_k(t)/(z-t)^2 dt;  z = u+iv, k = 0, ..., p
 *                             |
 *                             / -1
 *  or
 *                            / 1
 *                            |
 *  fac * 0.5*(1-x^2)^(1/2) * | P_k(t)/(x-t)^2 dt; |x| < 1 real,k = 0,...,p
 *                            |
 *                            / -1
 *
 *   where P_k(t) are the Legendre polynomials.
 *
 */
void 
q1(double u, double v, int const p, double const fac, double *Vre, double *Vim);


/* qn
 *
 * INPUT:	z       (point of evaluation)
 *          p       (max. polynomial degree)
 *          n       (upper limit of calculation of qn)
 *          fac     (factor to multiply)
 *          Vtmore  (temporary variables)
 *          Vtmpim  (temporary variables)
 * OUTPUT:  Vre     (real part of the integral)
 *          Vim     (imaginary part of the integral)
 *
 *                             / 1
 *                             |
 *  fac * -0.5*(z^2-1)^(1/2) * | P_k(t)/(z-t)^m dt;  z = u+iv, k = 0, ..., p
 *                             |
 *                             / -1
 *  or
 *                            / 1
 *                            |
 *  fac * 0.5*(1-x^2)^(1/2) * | P_k(t)/(x-t)^m dt; |x| < 1 real,k = 0,...,p
 *                            |
 *                            / -1
 *
 *  for m = 0..n and where P_k(t) are the Legendre polynomials.
 *
 */
void 
qn(fcomplex const z, int const p, int const n,  double const fac, double *Vtmpre, double *Vtmpim, double *Vre, double *Vim);


/* r0
 *
 * INPUT:	u,v     (point of evaluation z = u+iv)
 *          fac     (factor to multiply)
 *          p       (max. polynomial degree)
 * OUTPUT:  Vre     (real part of the integral)
 *          Vim     (imaginary part of the integral)
 *
 *         / 1
 *         |   N_k(t)
 *   fac * |  -------- dt;   z = u+iv, k = 1, ..., p
 *         |    z-t
 *         / -1
 *
 *                  / t
 *   where N_k(t) = |    P_(k-2)(xsi) dxsi
 *               -1 /
 *
 *
 */
void 
r0(double u, double v, int const p, double const fac, double *Vre, double *Vim);


/* r1
 *
 * INPUT:	u,v     (point of evaluation z = u+iv)
 *          fac     (factor to multiply)
 *          p       (max. polynomial degree)
 * OUTPUT:  Vre     (real part of the integral)
 *          Vim     (imaginary part of the integral)
 *
 *         / 1
 *         |     N_k(t)
 *   fac * |  ----------- dt;   z = u+iv, k = 1, ..., p
 *         |    (z-t)^2
 *         / -1
 *
 *                  / t
 *   where N_j(t) = |    P_(k-2)(xsi) dxsi 
 *               -1 /
 *
 *
 */
void 
r1(double u, double v, int const p, double const fac, double *Vre, double *Vim);
