/*************************************************************************/
/* Mex function for qtm1                                                 */
/*                                                                       */
/* AUTHORS: Markus Bantle                                                */
/* DATE: 04-20-2010                                                      */
/* Calculates the integrals qtm1 for complex points z up to pol. degree k*/
/*                                                                       */
/* INPUT:  list of complex points z = z(i,:) = [x(i), y(i)]              */
/*	       polynomial degree k                                           */
/*	       factor fac                                                    */
/*                                                                       */
/* OUTPUT: Matrix V =  Matrix V: each column contains the values for     */
/*                               one point                               */
/*                                                                       */
/*************************************************************************/

#include "mex.h"
#include "liblegfct.h"
#include "complex.h"
#include <string.h>

#ifdef _OPENMP
#include <omp.h>
#endif

void
mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
  int j,k,mz,nz, mk, nk, mf,nf;
  double *kptr, *z_re, *z_im, *fac, *Vre, *Vim;
#ifdef _OPENMP 
  int num_th;  
#endif
  
  if((nrhs<3)||(nrhs>4)){ 
      mexErrMsgTxt("Three inputs required:  z, k,fac");
  }
  if(nlhs>1){ 
      mexErrMsgTxt("One output required: V");
  }
    
  mz=mxGetM(prhs[0]);  nz=mxGetN(prhs[0]);
  mk=mxGetM(prhs[1]);  nk=mxGetN(prhs[1]);
  mf=mxGetM(prhs[2]);  nf=mxGetN(prhs[2]);

  if((nk!=1)||(mk!=1)){
      mexErrMsgTxt("Second argument (pol. degree) must be scalar");
  }
  if(nz!=1){
      mexErrMsgTxt("First Argument (z) must be of size numPoints x 1"); 
  }
  if((nf!=1)||((mf!=1)&&(mf!=mz))){
      mexErrMsgTxt("Factor has to be scalar or of size numPoints x 1");
  }
  
  z_re = mxGetPr(prhs[0]); 
  if (mxIsComplex(prhs[0])){
     z_im = mxGetPi(prhs[0]);
  }
  else {
     z_im = (double*) malloc(mz*sizeof(double));
     memset(z_im, 0, mz*sizeof(double));
  }
  
  kptr = mxGetPr(prhs[1]); 
  k = (int) *kptr;
  fac = (double*) mxGetPr(prhs[2]);
  
  plhs[0] = mxCreateNumericMatrix(k+1,mz,mxDOUBLE_CLASS, mxCOMPLEX);
  Vre = mxGetPr(plhs[0]);
  Vim = mxGetPi(plhs[0]);
  
#ifdef _OPENMP   
 if((mz<MZ_SWITCH)&&(k<K_SWITCH)){
    num_th = 1;
 } else{
    num_th = 6; /*omp_get_max_threads()-1*/;
 }
#endif

 if(mf==1){
   if(fabs(*fac)<1e-16){
     memset(Vre, 0, (k+1)*mz*sizeof(double)); 
     memset(Vim, 0, (k+1)*mz*sizeof(double)); 
     return;
   }
   #pragma omp parallel for num_threads(num_th)
   for(j=0; j<mz; j++){
     qtm1(z_re[j],z_im[j], k, fac[0], Vre+j*(k+1), Vim+j*(k+1));  
   }
 }else{
   #pragma omp parallel for num_threads(num_th)
   for(j=0; j<mz; j++){
     qtm1(z_re[j],z_im[j], k, fac[j], Vre+j*(k+1), Vim+j*(k+1));
   }
 }
}
