% buildV  computes the Galerkin matrix for the single layer potential
%         for the Laplace problem in 2D.
%
%   V = buildV(coordinates,elements,p[,options])
%   computes the Galerkin matrix for the single layer potential for a
%   boundary \Gamma specified by coordinates and elements and for a test 
%   and ansatz space with polynomial degrees p0 (either a non-negative 
%   integer or a vector of non-negative integers of length 
%   size(elements,1)) .
%   
%   Example:
%
%   addpath('../../general/c')
%   coordinates = [-1,0; -0.5,0; 0,0; 1,0];
%   elements    = [1,2; 2,3; 3,4];
%   p = [1,0,2];
%   V = buildV(coordinates,elements,p)
%
%   V =
%
%     8.7263e-02            0   3.2104e-02  -1.4881e-02  -1.1156e-02   9.5870e-04
%              0   9.9472e-03   8.1395e-03   5.6807e-03  -8.0925e-04   1.3948e-04
%     3.2104e-02   8.1395e-03   8.7263e-02   3.2978e-02  -2.1402e-02   3.8752e-03
%    -1.4881e-02   5.6807e-03   3.2978e-02   2.3873e-01            0  -1.3263e-02
%    -1.1156e-02  -8.0925e-04  -2.1402e-02            0   3.9789e-02            0
%     9.5870e-04   1.3948e-04   3.8752e-03  -1.3263e-02            0   1.3263e-02

function V = buildV(coord,elem,p,varargin)

if nargin <3
  error('At least three input arguments required.')
end
p = checkp0(p,size(elements,1));


ngauss = 32;
while ngauss<2*p
  ngauss = 2*ngauss;
end

nElem = size(elem,1);

[wg,xg] = gauss(ngauss);
pleg    = bLegendre(xg,max(p));

%*** for each element calculate u and m
u = (coord(elem(:,2),:) - coord(elem(:,1),:))/2;
m = (coord(elem(:,1),:) + coord(elem(:,2),:))/2;

%*** mesh width
h2 = sum(u.^2,2);
hh = sqrt(h2);

%*** cumulative sum of p; for the indices
cp = cumsum([0;p(:)+1]);

V = zeros(sum(p+1));
W = repmat(wg(:),1,max(p)+1);
for j = 1:nElem
  
  index1 = cp(j)+1:cp(j+1);
  
  % For identical elements we have an analytic formula
  V(index1,index1) = galVid(p(j),h2(j)*4);
    
  for k = j+1:nElem
    index2 = cp(k)+1:cp(k+1);
    if(norm(u(j,:))>norm(u(k,:))) %switch elements 
       % Transform Gauss points to Element k 
       points = 0; %TODO
       
       % DO THE QUADRATURE - IF POSSIBLE VECTORIZED
       
       Vsub = 0;
    else 
      % Transform the Gauss points to element j
      points = 0; % TODO
       
      % DO THE QUADRATURE - IF POSSIBLE VECTORIZED
      
      Vsub = 0;
    end 
     
    V(index1,index2) = Vsub;
    V(index2,index1) = Vsub';
  end
end


function V = galVid(p,h2)
V = zeros(p+1,p+1);
fac = -h2/(8*pi);
V(1,1) = fac*(2*log(h2)-6);
if p>=1
  % diagonal enries
  for i=2:p+1
    V(i,i) = -2*fac/(i*(i-1));  
  end
  % off-diagonal entries
  for j = 1:2:p-1     % first row
    V(1,j+2) = (fac*8)/(j*(j+1)*(j+2)*(j+3));
  end
  for i = 2:p+1       % remaining part of upper triangular part
    for k = 1:floor((p-i+1)/2)
      V(i,i+2*k) = 2*fac/((i+k-1)*(i+k)*(2*k-1)*(2*k+1));
    end
  end
  for i = 1:p+1       % symmetric part
    for j = i+2:2:p+1
      V(j,i) = V(i,j);
    end
  end
end


% checkp0 Ensure that p0 has the right size.
% 
%    p0 = checkp0(p0,N) 
%    Ensures that p0 is a vector of size (N,1) containing only non-negative
%    values. 
%    If the input argument p0 is a scalar, the output is a vector of size
%    (N,1) where all values are that scalar.
function p0 = checkp0(p0,N)

if length(p0) == 1 && p0>=0
  p0 = p0*ones(N,1);
elseif numel(p0) == N && min(p0)>=0
  p0 = p0(:);
else
  error('p0 must be a vector of length size(elements,1) or a non-negative integer.')
end
