% showSolDom   Use the representation formula to evaluate and display the
%              solution in the interior or exterior domain. 
%
%    showSolDom(coordinates,elements,u,p1,phi,p0,sx, sy, domain, options)
%    Uses the Cauchy data u and the Neumann data phi given by the coefficient 
%    vectors for the respective ansatz spaces (with polynomial degrees p0 for 
%    phi and p1 for u) to calculate the solution in the domain specified by 
%    input argument domain. 
%    The boundary \Gamma of the domain is discretized by coordinates and
%    elements.
%    The grid on which the solution is evaluated and displayed is determined by the 
%    vectors sx and sy. 
%
% Example:
%  elements    = [1,2; 2,3; 3,4; 4,1];
%  coordinates = [0,0; 1,0; 1,1; 0,1];
%  u   =  ones(4,1);
%  phi =  ones(4,1);
%  p0  =  0;
%  p1  =   1;
%  sx  = linspace(-2,3,50); sy = sx;
%  domain = 'exterior';
%  addpath('../laplace/mat') 
%  addpath('./c')
%  show_sol_dom(coordinates,elements,u,p1,phi,p0,sx, sy, domain);

function [X,Y,Z] = showSolDom(coordinates,elements,u,p1,phi,p0,sx, sy,...
  domain, varargin)

if nargin < 9
  error ('At least 9 input arguments are required.')
else
  options = [];
  for k = 1:nargin-9
    if isstruct(varargin{k})
      options = varargin{k};
    else
      disp(['Warning: didn''t recognize Argument ',num2str(k+9),'.']);
    end
  end
end

if size(elements,1)==size(coordinates,1)  % closed boundary
  % compute interior and exterior
  [X,Y,M] = chkinout(sx,sy,coordinates,elements);
  Z = zeros(size(X));
  if isempty(domain), domain='interior'; end
  switch domain
    case 'exterior'
      index = find(M==0);
      Z(M>0) = NaN;
    otherwise
      index = find(M);
      Z(M==0) = NaN;
  end
else
  [X,Y] = meshgrid(sx,sy);
  Z = zeros(size(X));
  index = 1:length(Z(:));
end

X1 = X(:);
Y1 = Y(:);
if( ~isempty(u))
  dimension = size(u,2);
else
  dimension = size(phi,2);
end
Z1 = repmat(Z(:),1,dimension);

if ~isempty(u) 
  if(numel(p1)==1)
    p1 = p1*ones(size(elements,1),1);
  end  
  cp1 = cumsum([size(coordinates,1);p1(:)-1]);  
      
  for k = 1:size(elements,1)
    locK = potK(coordinates(elements(k,:),:),[X1(index),Y1(index)],p1(k),options); 
    index2 = [elements(k,:),cp1(k)+1:cp1(k+1)]';
    locu = u(index2,:);
    Z1(index,:) = Z1(index,:)-reshape(locK*locu(:), numel(index),dimension);
  end
end

if ~isempty(phi) 
  if(numel(p0)==1)
    p0 = p0*ones(size(elements,1),1);
  end
  cp0 = cumsum([0;p0(:)+1]);  
  for k = 1:size(elements,1)
    locV = potV(coordinates(elements(k,:),:),[X1(index),Y1(index)],p0(k),options);
    index2 = (cp0(k)+1:cp0(k+1))';   
    locphi = phi(index2,:);
    Z1(index,:) = Z1(index,:)+reshape(locV*locphi(:),numel(index),dimension);
  end
end

for d = 1:dimension
  Z = reshape(Z1(:,d),size(Z));
  subplot(dimension,1,d)
  surf(X,Y,Z)
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @function   chkinout.m verifies, if points are contained in a polygon 
% @param:     xx          -   vector of points for x axis
%             yy          -   vector of points for y axis
%             coordinates -   vector of vertices
%             edges       -   edges are NOT ordered
% @return:    X           -   see meshgrid
%             Y           -   see meshgrid
%             Z           -   logical matrix, 1 -> point is in set
%                                             0 -> point is NOT in set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ X , Y , Z ] = chkinout( xx , yy , coordinates , edges )

c = find_comp(edges);
coordinates_sorted0=[0,0];
coordinates_sorted1=[3,5];
for i=1:length(c)
  coordinates_sorted0 = [coordinates_sorted0; coordinates(c{i},:);0,0];
  coordinates_sorted1 = [coordinates_sorted1; coordinates(c{i},:);3,5];
end

%calculate bounding box, Z(i,j) = 1 if in boundig box, else Z(i,j) = 0
xymin = min(coordinates,[],1);
xymax = max(coordinates,[],1);

zx = find( ( xx(:) >= xymin(1) ) & ( xx(:) <= xymax(1) ) );
zy = find( ( yy(:) >= xymin(2) ) & ( yy(:) <= xymax(2) ) );
[ X , Y ] = meshgrid( xx(zx) , yy(zy) );

warning off
pin0=inpoly(coordinates_sorted0(:,1),coordinates_sorted0(:,2),X(:),Y(:));
pin1=inpoly(coordinates_sorted1(:,1),coordinates_sorted1(:,2),X(:),Y(:));
warning on
pin = pin0|pin1;

Z = zeros(length(yy),length(xx));
Z(zy,zx) = reshape(pin,length(zy),length(zx));
if nargout == 3
  [X,Y] = meshgrid( xx(:) , yy(:) );
end

function c = find_comp(elements)
used = ones(size(elements,1),1);
map(elements(:,1))=1:size(elements,1);
anz_comp = 0;
while any(used)
  i = find(used);
  i = i(1);
  comp = elements(i,:);
  a = comp(1);
  next = comp(2);
  used(i) = 0;
  while a~=next
    i = map(next);
    comp = [comp;elements(i,:)];
    next = comp(end,2);
    used(i) = 0;
  end
  anz_comp = anz_comp + 1;
  c{anz_comp} = [comp(:,1);comp(1,1)];
end

